/*
 * HTMLOptionPane.java
 * Copyright (c) 2000 Andre Kaplan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import org.gjt.sp.jedit.jEdit;
import org.gjt.sp.jedit.AbstractOptionPane;
import org.gjt.sp.jedit.GUIUtilities;
import org.gjt.sp.jedit.gui.HistoryModel;
import org.gjt.sp.jedit.gui.HistoryTextField;


public class HTMLOptionPane extends AbstractOptionPane
{
    private JCheckBox        showTagHighlightDefault;
    private JCheckBox        useTagHighlightFilter;
    private HistoryTextField tagHighlightFilter;
    private JButton          tagHighlightColor;

    public HTMLOptionPane() {
        super("html");
    }


    public void _init() {
        this.showTagHighlightDefault = this.createCheckBox("html.show-tag-highlight-default", true);
        this.useTagHighlightFilter   = this.createCheckBox("html.use-tag-highlight-filter", true);
        this.tagHighlightFilter = new HistoryTextField("html.tag-highlight-filter", true);
        this.tagHighlightColor  = this.createColorButton("html.tag-highlight-color");

        this.useTagHighlightFilter.addActionListener(new UseFilterHandler());

        HistoryModel filterHistory = HistoryModel.getModel("html.tag-highlight-filter");
        String filter;
        if  (filterHistory.getSize() > 0) {
            filter = filterHistory.getItem(0);
        } else {
            filter = jEdit.getProperty("html.tag-highlight-default-filter", "*.{html,htm}");
        }
        this.tagHighlightFilter.setText(filter);
        this.tagHighlightFilter.addCurrentToHistory();
        this.tagHighlightFilter.setEnabled(this.useTagHighlightFilter.isSelected());

        addComponent(this.showTagHighlightDefault);
        addComponent(this.useTagHighlightFilter);
        addComponent(jEdit.getProperty("options.html.tag-highlight-filter"),
            this.tagHighlightFilter);
        addComponent(jEdit.getProperty("options.html.tag-highlight-color"),
            this.tagHighlightColor);
    }


    public void _save() {
        jEdit.setBooleanProperty("html.show-tag-highlight-default",
            this.showTagHighlightDefault.isSelected());
        jEdit.setBooleanProperty("html.use-tag-highlight-filter",
            this.useTagHighlightFilter.isSelected());
        if (this.useTagHighlightFilter.isSelected()) {
            this.tagHighlightFilter.addCurrentToHistory();
        }
        jEdit.setProperty("html.tag-highlight-color",
            GUIUtilities.getColorHexString(this.tagHighlightColor.getBackground())
        );
    }


    private JButton createColorButton(String property) {
        JButton b = new JButton(" ");
        b.setBackground(GUIUtilities.parseColor(jEdit.getProperty(property)));
        b.addActionListener(new ColorButtonHandler(b));
        b.setRequestFocusEnabled(false);
        return b;
    }


    private JCheckBox createCheckBox(String property, boolean defaultValue) {
        JCheckBox cb = new JCheckBox(jEdit.getProperty("options." + property));
        cb.setSelected(jEdit.getBooleanProperty(property, defaultValue));
        return cb;
    }


    private class ColorButtonHandler implements ActionListener {
        private JButton button;


        public ColorButtonHandler(JButton button) {
            this.button = button;
        }


        public void actionPerformed(ActionEvent evt) {
            JButton button = (JButton)evt.getSource();
            Color c = JColorChooser.showDialog(HTMLOptionPane.this,
                jEdit.getProperty("colorChooser.title"),
                button.getBackground()
            );
            if (c != null) {
                button.setBackground(c);
            }
        }
    }


    private class UseFilterHandler implements ActionListener {
        public UseFilterHandler() {}


        public void actionPerformed(ActionEvent evt) {
            HTMLOptionPane.this.tagHighlightFilter.setEnabled(
                HTMLOptionPane.this.useTagHighlightFilter.isSelected()
            );
        }
    }
}
