/*
 * chars_to_entities.java
 * Copyright (C) 1999-2000 Romain Guy, Andre Kaplan
 *
 * This free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


import org.gjt.sp.jedit.EditAction;
import org.gjt.sp.jedit.View;
import org.gjt.sp.jedit.textarea.JEditTextArea;

import org.gjt.sp.util.Log;

import javax.swing.text.Segment;


public class chars_to_entities extends EditAction
{
    private final boolean selection;


    public chars_to_entities() {
        super("chars-to-entities-" + ((false) ? "selection" : "buffer"));
        this.selection = false;
    }


    public chars_to_entities(boolean selection) {
        super("chars-to-entities-" + ((selection) ? "selection" : "buffer"));
        this.selection = selection;
    }


    public void invoke(View view) {
        JEditTextArea textArea = view.getTextArea();

        if (this.selection) {
            String text = textArea.getSelectedText();
            if (text == null) {
                textArea.getToolkit().beep();
                return;
            }
            textArea.setSelectedText(this.doEntities(text));
        } else {
            Segment segment = new Segment();
            textArea.getText(0, textArea.getBufferLength(), segment);
            textArea.setText(this.doEntities(segment));
        }
    }


    private String doEntities(String html) {
        return this.doEntities(html.toCharArray(), 0, html.length());
    }


    private String doEntities(Segment html) {
        return this.doEntities(html.array, html.offset, html.count);
    }


    private String doEntities(char[] str, int strOff, int strLen) {
        StringBuffer buf = new StringBuffer();
        char c;
        int len = 0;
        int off = strOff;
        for (int i = 0; i < strLen; i++) {
            c = str[strOff + i];

            String entity = HTMLEntity.lookupEntity((short) c);
            if (entity != null) {
                buf.append(str,off,len).append("&").append(entity).append(";");
                off += len + 1; len = 0;
            } else if (((int) c) > 255) {
                buf.append(str,off,len).append("&#").append((int)c).append(";");
                off += len + 1; len = 0;
            } else {
                len++;
            }
        }

        buf.append(str, off, len);
        return buf.toString();
    }
}

