/*
 * close_last_open_element.java
 * Copyright (C) 1998, 2000 Ollie Rutherfurd <oliver@rutherfurd.net>, Andre Kaplan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


import org.gjt.sp.jedit.textarea.JEditTextArea;
import org.gjt.sp.jedit.EditAction;
import org.gjt.sp.jedit.View;
import javax.swing.text.BadLocationException;
import javax.swing.text.Segment;
import java.util.Stack;

import org.gjt.sp.util.Log;


/**
 * Closes the open HTML/XML element
 * @author Ollie Rutherfurd
 * @author Andre Kaplan
**/
public class close_last_open_element extends EditAction
{
    public close_last_open_element()
    {
        super("close-last-open-element");
    }


    public void invoke(View view)
    {
        JEditTextArea textArea = view.getTextArea();
        int caretPos = textArea.getCaretPosition();
        Segment startOfBuffer = new Segment();
        textArea.getText(0, caretPos, startOfBuffer);

        String tag = findLastOpenTag(startOfBuffer);

        if(tag.length() > 0) {
            insertTag(textArea, tag);
        } else {
            view.getToolkit().beep();
        }
    }


    public boolean isRepeatable()
    {
        return false;
    }


    /*
    *   returns the name of a tag (stops at first space)
    */
    private String getTagName(char[] buf, int offset, int len) {
        int startIdx = offset;
        int endIdx   = offset + len;

        for (int i = startIdx; i < endIdx; i++) {
            if (!Character.isWhitespace(buf[i])) {
                startIdx = i;
                break;
            }
        }

        for (int i = startIdx + 1; i < endIdx; i++) {
            if (Character.isWhitespace(buf[i])) {
                endIdx = i;
                break;
            }
        }

        return new String(buf, startIdx, endIdx - startIdx);
    }


    /*
    *   finds the last tag which hasn't been closed
    *   this includes tags such as <img> or <br>
    *
    *   note: it doesn't check whether tags match
    *   eg <p><b><i></b></i> (will find <p>)
    *
    */
    private String findLastOpenTag(Segment s){
        Stack tagStack = new Stack();
        int tagEnd = 0;
        int curOff = s.count - 1;
        int curPos = s.offset + s.count - 1;
        boolean inTag = false;
        char lastChar = '*';
        char curChar = '*';
        String curTag = "";

        while (curOff >= 0) {
            curChar = s.array[curPos];

            // start of tag (or end)
            if(curChar == '>'){
                tagEnd = curPos;
                inTag = true;
            }
            // if tag is something like '<br />', don't push onto stack, just skip
            else if (curChar == '/') {

                if (lastChar == '>') {
                    inTag = false;
                }
            }
            // if in tag either push of pop a tag, depending on whether
            // last char as a wack '/' or not
            else if (curChar == '<') {

                if (lastChar == '?') {
                    inTag = false;
                }

                if (lastChar == '!') {
                    inTag = false;
                }

                if (inTag == true) {
                    // if closing a tag, push it onto the stack
                    if (lastChar == '/') {
                        curTag = getTagName(s.array, curPos + 2, tagEnd - (curPos + 2));
                        tagStack.push(curTag);
                    } else {
                        curTag = getTagName(s.array, curPos + 1, tagEnd - (curPos + 1));

                        if (tagStack.empty()) {
                            return curTag;
                        }

                        String lastTag = (String) tagStack.peek();
                        if (lastTag.equalsIgnoreCase(curTag)) {
                            tagStack.pop();
                        }
                    }
                }
            }

            lastChar = s.array[curPos];
            curPos--;
            curOff--;
        }

        return "";
    }


    /*
    *   inserts tag into textarea at current position
    */
    private void insertTag(JEditTextArea textArea, String tag){
        int caretPos = textArea.getCaretPosition();
        int selStart = textArea.getSelectionStart();
        int selEnd = textArea.getSelectionEnd();
        int offset = tag.length() + 3;

        if(tag.length() > 0){

            textArea.setSelectionStart(caretPos);
            textArea.setSelectionEnd(caretPos);

            textArea.setSelectedText("</" + tag + ">");

            if(caretPos == selStart && caretPos < selEnd){
                textArea.setCaretPosition(caretPos + offset);
                textArea.setSelectionEnd(selEnd + offset);
                textArea.setSelectionStart(selStart + offset);
            }
            else{
                textArea.setCaretPosition(caretPos);
                textArea.setSelectionStart(selStart);
                textArea.setSelectionEnd(selEnd);
            }
        }
    }
}

