/*
 * entities_to_chars.java
 * Copyright (C) 1999-2000 Romain Guy, Andre Kaplan
 *
 * This free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


import org.gjt.sp.jedit.EditAction;
import org.gjt.sp.jedit.View;
import org.gjt.sp.jedit.textarea.JEditTextArea;

import java.util.Hashtable;


public class entities_to_chars extends EditAction
{
    private final boolean selection;


    public entities_to_chars() {
        super("entities-to-chars-" + ((false) ? "selection" : "buffer"));
        this.selection = false;
    }


    public entities_to_chars(boolean selection) {
        super("entities-to-chars-" + ((selection) ? "selection" : "buffer"));
        this.selection = selection;
    }


    public void invoke(View view) {
        JEditTextArea textArea = view.getTextArea();

        if (this.selection) {
            String text = textArea.getSelectedText();
            if (text == null) {
                textArea.getToolkit().beep();
                return;
            }
            textArea.setSelectedText(this.doChars(text));
        } else {
            textArea.setText(this.doChars(textArea.getText()));
        }
    }


    private String doChars(String html) {
        StringBuffer buf = new StringBuffer();
        int entityOff = -1;

        for (int i = 0; i < html.length(); i++)
        {
            char c = html.charAt(i);
            switch (c)
            {
            case '&':
                if (entityOff == -1) {
                    entityOff = i;
                }
                break;

            case ';':
                if (entityOff != -1) {
                    String entity = html.substring(entityOff + 1, i);
                    if (entity.charAt(0) == '#') { // Numeric entity
                        try {
                            int ch = Integer.parseInt(entity.substring(1));
                            if (    (ch >= (int) Character.MIN_VALUE)
                                &&  (ch <= (int) Character.MAX_VALUE)
                            ) {
                                buf.append((char) ch);
                            } else {
                                buf.append('&').append(entity).append(';');
                            }
                        } catch (NumberFormatException nfe) {
                            buf.append('&').append(entity).append(';');
                        }
                    } else { // Character Entity
                        short ch = HTMLEntity.lookupCode(entity);
                        if (ch == 0) {
                            buf.append('&').append(entity).append(';');
                        } else {
                            buf.append((char) ch);
                        }
                    }

                    entityOff = -1;
                    break;
                }

            default:
                if (entityOff == -1) {
                    buf.append(c);
                }
                break;
            }
        }

        return buf.toString();
    }
}

