/*
 * jtidy_generate_config_file.java
 * Copyright (c) 2000 Andre Kaplan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


import java.io.IOException;
import java.io.OutputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;

import java.util.Properties;

import org.gjt.sp.jedit.Buffer;
import org.gjt.sp.jedit.EBComponent;
import org.gjt.sp.jedit.EBMessage;
import org.gjt.sp.jedit.EditAction;
import org.gjt.sp.jedit.EditBus;
import org.gjt.sp.jedit.EditPane;
import org.gjt.sp.jedit.jEdit;
import org.gjt.sp.jedit.View;

import org.gjt.sp.jedit.msg.BufferUpdate;
import org.gjt.sp.jedit.msg.EditPaneUpdate;

import org.gjt.sp.util.Log;


public class jtidy_generate_config_file
    extends EditAction
    implements EBComponent
{
    public jtidy_generate_config_file() {
        super("jtidy-generate-config-file");
    }


    public void invoke(View view) {
        String res = "";

        try {
            Properties props = new Properties();
            HTMLPlugin.saveTo(props);

            ByteArrayOutputStream baOut = new ByteArrayOutputStream();
            OutputStream out = new NewlineOutputFilter(new BufferedOutputStream(baOut));

            props.save(out, "Generated Tidy configuration file");
            // JDK1.2+
            // props.store(out, "Generated Tidy configuration file");

            out.flush();
            out.close();

            res = new String(baOut.toByteArray());
        } catch (IOException ioe) {
            Log.log(Log.ERROR, this, ioe);
            return;
        }

        EditBus.addToBus(this);
        this.job = new FillBuffer(view.getEditPane(), res);
        Buffer buffer = jEdit.newFile(view);
    }


    public void handleMessage(EBMessage message) {
        if (message instanceof BufferUpdate) {
            BufferUpdate bu = (BufferUpdate) message;
            if (bu.getWhat() == BufferUpdate.CREATED) {
                if (this.job != null) {
                    this.job.setBuffer(bu.getBuffer());
                }
            }
        } else if (message instanceof EditPaneUpdate) {
            EditPaneUpdate epu = (EditPaneUpdate) message;
            if (epu.getWhat() == EditPaneUpdate.BUFFER_CHANGED) {
                if (this.job != null) {
                    this.job.run();
                    this.job = null;
                }
                EditBus.removeFromBus(this);
            }
        }
    }


    private FillBuffer job;


    private static class FillBuffer
    {
        private Buffer   buffer;
        private EditPane editPane;
        private String   text;


        private FillBuffer() {}


        public FillBuffer(EditPane editPane, String text) {
            this.editPane = editPane;
            this.text     = text;
        }


        public void run() {
            if (this.buffer != this.editPane.getBuffer()) {
                Log.log(Log.DEBUG, this, "buffer != editPane.getBuffer()");
            } else {
                this.editPane.getTextArea().setText(this.text);
            }
        }


        public void setBuffer(Buffer buffer) {
            this.buffer = buffer;
        }
    }
}

