/*
 * remove_tags.java
 * Copyright (C) 1999 Romain Guy
 *
 * This	free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


import org.gjt.sp.jedit.Buffer;
import org.gjt.sp.jedit.EditAction;
import org.gjt.sp.jedit.MiscUtilities;
import org.gjt.sp.jedit.View;
import org.gjt.sp.jedit.textarea.JEditTextArea;

import javax.swing.text.Element;
import javax.swing.text.BadLocationException;


public class remove_tags extends EditAction {
    public remove_tags() {
        super("remove-tags");
    }


    public void invoke(View view) {
        Buffer buffer = view.getBuffer();
        JEditTextArea textArea = view.getTextArea();
        buffer.beginCompoundEdit();
        Element lineElement;
        Element map = buffer.getDefaultRootElement();
        int index = map.getElementIndex(0);
        int count = map.getElementCount();
        for (int i = index; i < count; i++) {
            lineElement = map.getElement(i);
            int start = lineElement.getStartOffset();
            int end = lineElement.getEndOffset() - 1;
            end -= start;
            try {
                String text = deleteTags(textArea.getText(start, end));
                buffer.remove(start, end);
                if (text != null && !isFullOfSpaces(text)) {
                    buffer.insertString(start, text.substring(MiscUtilities.getLeadingWhiteSpace(text)), null);
                }
            } catch (BadLocationException ble) { }
        }
        buffer.endCompoundEdit();
    }


    private boolean isFullOfSpaces(String in) {
        for (int i = 0; i < in.length(); i++) {
            if (in.charAt(i) != ' ' || in.charAt(i) != '\t') { return false; }
        }
        return true;
    }


    private String deleteTags(String in) {
        char c;
        int spe = -1;
        int tag = -1;
        StringBuffer buf = new StringBuffer(in.length());
        for (int i = 0; i < in.length(); i++) {
            switch ((c = in.charAt(i))) {
            case '<':
                tag = i;
                break;

            case '>':
                if (tag != -1) {
                    if (in.substring(tag + 1, i).equalsIgnoreCase("BR")) {
                        buf.append('\n');
                    } else if (in.substring(tag + 1, i).equalsIgnoreCase("P")) {
                        buf.append('\n').append('\n');
                    }
                    tag = -1;
                }
                break;

            case '&':
                if (tag == -1) { spe = i; }
                break;

            case ';':
                if (spe != -1) {
                    String spc = in.substring(spe, i);
                    if (spc.equals("&nbsp")) {
                        buf.append(' ');
                    } else if (spc.equals("&quot")) {
                        buf.append('\"');
                    } else if (spc.equals("&gt")) {
                        buf.append('>');
                    } else if (spc.equals("&lt")) {
                        buf.append('<');
                    } else if (spc.equals("&amp")) {
                        buf.append('&');
                    }
                    spe = -1;
                    break;
                }
                if (tag == -1) { buf.append(';'); }
                break;

            default:
                if (tag == -1 && spe == -1) {
                    buf.append(c);
                }
                break;
            }
        }
        return buf.toString();
    }
}

